unit spl_base;

interface

uses
  Windows,
  spl_com,
  spl_arc,
  spl_menu,
  spl_thum,
  spl_gen,
  spl_view,
  spl_fs,
  spl_gui,
  spl_xpth,
  spl_file;

{$if defined(_DEBUG) and defined(TRACE_ENABLE)}
  procedure memcpy _sal_safe_memcpy(dest: Pointer; src: Pointer; count: size_t);
{$ifend}

{$HPPEMIT '#if (FILE_ATTRIBUTE_ENCRYPTED != 0x00004000)'}
{$HPPEMIT '#pragma message(__FILE__ " ERROR: FILE_ATTRIBUTE_ENCRYPTED != 0x00004000. You have to install latest version of Microsoft SDK. This value has changed!")'}
{$HPPEMIT '#endif'}

const
  MENU_EVENT_TRUE = $0001;
  MENU_EVENT_DISK = $0002;
  MENU_EVENT_THIS_PLUGIN_ARCH = $0004;
  MENU_EVENT_THIS_PLUGIN_FS = $0008;
  MENU_EVENT_FILE_FOCUSED = $0010;
  MENU_EVENT_DIR_FOCUSED = $0020;
  MENU_EVENT_UPDIR_FOCUSED = $0040;
  MENU_EVENT_FILES_SELECTED = $0080;
  MENU_EVENT_DIRS_SELECTED = $0100;
  MENU_EVENT_TARGET_DISK = $0200;
  MENU_EVENT_TARGET_THIS_PLUGIN_ARCH = $0400;
  MENU_EVENT_TARGET_THIS_PLUGIN_FS = $0800;
  MENU_EVENT_SUBDIR = $1000;
  MENU_EVENT_ARCHIVE_FOCUSED = $2000;

  MENU_SKILLLEVEL_BEGINNER = $0001;
  MENU_SKILLLEVEL_INTERMEDIATE = $0002;
  MENU_SKILLLEVEL_ADVANCED = $0004;
  MENU_SKILLLEVEL_ALL = $0007;

  PLUGINEVENT_COLORSCHANGED = 0;
  PLUGINEVENT_CONFIGURATIONCHANGED = 1;
  PLUGINEVENT_PANELSSWAPPED = 2;
  PLUGINEVENT_PANELACTIVATED = 3;

  FUNCTION_PANELARCHIVERVIEW = $0001;
  FUNCTION_PANELARCHIVEREDIT = $0002;
  FUNCTION_CUSTOMARCHIVERPACK = $0004;
  FUNCTION_CUSTOMARCHIVERUNPACK = $0008;
  FUNCTION_CONFIGURATION = $0010;
  FUNCTION_LOADSAVECONFIGURATION = $0020;
  FUNCTION_VIEWER = $0040;
  FUNCTION_FILESYSTEM = $0080;
  FUNCTION_DYNAMICMENUEXT = $0100;

  LOADINFO_INSTALL = $0001;
  LOADINFO_NEWSALAMANDERVER = $0002;
  LOADINFO_NEWPLUGINSVER = $0004;
  LOADINFO_LOADONSTART = $0008;

  SALHOTKEY_HINT = $00020000;

type
{$if (defined(_DEBUG) or defined(CALLSTK_MEASURETIMES)) and not defined(CALLSTK_DISABLEMEASURETIMES) }
  TCallStackMsgContext = packed record
    PushesCounterStart: DWORD;
    PushPerfTimeCounterStart: LARGE_INTEGER;
    IgnoredPushPerfTimeCounterStart: LARGE_INTEGER;
    StartTime: LARGE_INTEGER;
    PushCallerAddress: DWORD;
  end;
{$else}
  TCallStackMsgContext = packed record
  end;
{$ifend}
  PCallStackMsgContext = ^TCallStackMsgContext;

  TCallStackFunc = function(param: Pointer): Cardinal; stdcall;

  TSalamanderDebugAbstract = class
    procedure TraceI(_file: PAnsiChar; line: Integer; str: PAnsiChar); virtual; stdcall; abstract;
    procedure TraceE(_file: PAnsiChar; line: Integer; str: PAnsiChar); virtual; stdcall; abstract;
    procedure TraceAttachThread(thread: THandle; tid: Cardinal); virtual; stdcall; abstract;
    procedure TraceSetThreadName(name: PAnsiChar); virtual; stdcall; abstract;
    function CallWithCallStack(threadBody: TCallStackFunc; param: Pointer): Cardinal; virtual; stdcall; abstract;
    procedure Push(format: PAnsiChar; args: va_list; callStackMsgContext: PCallStackMsgContext; doNotMeasureTimes: BOOL); virtual; stdcall; abstract;
    procedure Pop(callStackMsgContext: PCallStackMsgContext); virtual; stdcall; abstract;
    procedure SetThreadNameInVC(name: PAnsiChar); virtual; stdcall; abstract;
    procedure SetThreadNameInVCAndTrace(name: PAnsiChar); virtual; stdcall; abstract;
    procedure TraceConnectToServer; virtual; stdcall; abstract;
    procedure SetThreadNameInVC_P(name: PAnsiChar); virtual; stdcall;
    procedure SetThreadNameInVCAndTrace_P(name: PAnsiChar); virtual; stdcall;
    procedure TraceConnectToServer_P; virtual; stdcall;
  end;

  TSalamanderConnectAbstract = class
    procedure AddCustomPacker(title: PAnsiChar; defaultExtension: PAnsiChar; update: BOOL); virtual; stdcall; abstract;
    procedure AddCustomUnpacker(title: PAnsiChar; masks: PAnsiChar; update: BOOL); virtual; stdcall; abstract;
    procedure AddPanelArchiver(extensions: PAnsiChar; edit: BOOL; updateExts: BOOL); virtual; stdcall; abstract;
    procedure AddViewer(masks: PAnsiChar; force: BOOL); virtual; stdcall; abstract;
    procedure ForceRemoveViewer(mask: PAnsiChar); virtual; stdcall; abstract;
    procedure AddMenuItem(iconIndex: Integer; name: PAnsiChar; hotKey: DWORD; id: Integer; callGetState: BOOL; state_or: DWORD; state_and: DWORD; skillLevel: DWORD); virtual; stdcall; abstract;
    procedure AddSubmenuStart(iconIndex: Integer; name: PAnsiChar; id: Integer; callGetState: BOOL; state_or: DWORD; state_and: DWORD; skillLevel: DWORD); virtual; stdcall; abstract;
    procedure AddSubmenuEnd; virtual; stdcall; abstract;
    procedure SetChangeDriveMenuItem(title: PAnsiChar; iconIndex: Integer); virtual; stdcall; abstract;
    procedure SetThumbnailLoader(masks: PAnsiChar); virtual; stdcall; abstract;
    procedure SetBitmapWithIcons(bitmap: HBITMAP); virtual; stdcall; abstract;
    procedure SetPluginIcon(iconIndex: Integer); virtual; stdcall; abstract;
    procedure SetPluginMenuAndToolbarIcon(iconIndex: Integer); virtual; stdcall; abstract;
    procedure SetIconListForGUI(iconList: TGUIIconListAbstract); virtual; stdcall; abstract;
    procedure SetIconListForGUI_P(iconList: TGUIIconListAbstract); virtual; stdcall;
  end;

  TDynamicString = class
    function Add(str: PAnsiChar; len: Integer = -1): BOOL; virtual; stdcall; abstract;
  end;

  TPluginInterfaceAbstract = class
    function GetVersion: Integer; virtual; stdcall; abstract;
    procedure About(parent: HWND); virtual; stdcall; abstract;
    function Release(parent: HWND; force: BOOL): BOOL; virtual; stdcall; abstract;
    procedure LoadConfiguration(parent: HWND; regKey: HKEY; registry: TSalamanderRegistryAbstract); virtual; stdcall; abstract;
    procedure SaveConfiguration(parent: HWND; regKey: HKEY; registry: TSalamanderRegistryAbstract); virtual; stdcall; abstract;
    procedure Configuration(parent: HWND); virtual; stdcall; abstract;
    procedure Connect(parent: HWND; salamander: TSalamanderConnectAbstract); virtual; stdcall; abstract;
    procedure ReleasePluginDataInterface(pluginData: TPluginDataInterfaceAbstract); virtual; stdcall; abstract;
    function GetInterfaceForArchiver: TPluginInterfaceForArchiverAbstract; virtual; stdcall; abstract;
    function GetInterfaceForViewer: TPluginInterfaceForViewerAbstract; virtual; stdcall; abstract;
    function GetInterfaceForMenuExt: TPluginInterfaceForMenuExtAbstract; virtual; stdcall; abstract;
    function GetInterfaceForFS: TPluginInterfaceForFSAbstract; virtual; stdcall; abstract;
    function GetInterfaceForThumbLoader: TPluginInterfaceForThumbLoaderAbstract; virtual; stdcall; abstract;
    procedure Event(event: Integer; param: DWORD); virtual; stdcall; abstract;
    procedure ClearHistory(parent: HWND); virtual; stdcall; abstract;
    procedure AcceptChangeOnPathNotification(path: PAnsiChar; includingSubdirs: BOOL); virtual; stdcall; abstract;
    function UninstallUnregisteredComponents(parent: HWND; componentsDescr: PAnsiChar; uninstallSPL: PBOOL; uninstallLangDir: PBOOL; pluginDir: PAnsiChar; deleteFileList: TDynamicString): BOOL; virtual; stdcall; abstract;
  end;

  TSalamanderPluginEntryAbstract = class
    function GetVersion: Integer; virtual; stdcall; abstract;
    function GetParentWindow: HWND; virtual; stdcall; abstract;
    function SetBasicPluginData(pluginName: PAnsiChar; supportPanelView: BOOL; supportPanelEdit: BOOL; supportCustomPack: BOOL; supportCustomUnpack: BOOL; supportConfiguration: BOOL; supportLoadSave: BOOL; supportViewer: BOOL; supportFS: BOOL; version: PAnsiChar; copyright: PAnsiChar; description: PAnsiChar; regKeyName: PAnsiChar; extensions: PAnsiChar; fsName: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function GetSalamanderDebug: TSalamanderDebugAbstract; virtual; stdcall; abstract;
    function SetBasicPluginData2(pluginName: PAnsiChar; functions: DWORD; version: PAnsiChar; copyright: PAnsiChar; description: PAnsiChar; regKeyName: PAnsiChar = nil; extensions: PAnsiChar = nil; fsName: PAnsiChar = nil): BOOL; virtual; stdcall; abstract;
    function GetSalamanderGeneral: TSalamanderGeneralAbstract; virtual; stdcall; abstract;
    function GetLoadInformation: DWORD; virtual; stdcall; abstract;
    procedure GetRegistrationKeyInfo(registeredVersion: PBOOL; productID1: PDWORD; productID2: PDWORD; productID3: PDWORD; numberOfLicenses: PInteger; neverExpires: PBOOL; expiration: PSystemTime; educationalLicense: PBOOL); virtual; stdcall; abstract;
    function LoadLanguageModule(parent: HWND; pluginName: PAnsiChar): HINST; virtual; stdcall; abstract;
    function GetCurrentSalamanderLanguageID: Word; virtual; stdcall; abstract;
    function GetSalamanderGUI: TSalamanderGUIAbstract; virtual; stdcall; abstract;
    function GetXpThemeSupport: TXpThemeSupportAbstract; virtual; stdcall; abstract;
    function GetSalamanderSafeFile: TSalamanderSafeFileAbstract; virtual; stdcall; abstract;
    procedure SetPluginHomePageURL(url: PAnsiChar); virtual; stdcall; abstract;
    function AddFSName(fsName: PAnsiChar; newFSNameIndex: PInteger): BOOL; virtual; stdcall; abstract;
    function ShowNotRegisteredDlg(pluginIcon: HICON; buyNowURL: PAnsiChar): BOOL; virtual; stdcall; abstract;
  end;

TSalamanderPluginEntryFunc = function(salamander: TSalamanderPluginEntryAbstract): TPluginInterfaceAbstract; stdcall;
TSalamanderPluginGetReqVerFunc = function: Integer; stdcall;
TSalamanderPluginGetSDKVer = function: Integer; stdcall;

function SALHOTKEY(vk: Byte; mods: Word): DWORD;

implementation

procedure TSalamanderDebugAbstract.SetThreadNameInVC_P(name: PAnsiChar);
begin
  SetThreadNameInVC(name);
end;

procedure TSalamanderDebugAbstract.SetThreadNameInVCAndTrace_P(name: PAnsiChar);
begin
  SetThreadNameInVCAndTrace(name);
end;

procedure TSalamanderDebugAbstract.TraceConnectToServer_P;
begin
  TraceConnectToServer;
end;

procedure TSalamanderConnectAbstract.SetIconListForGUI_P(iconList: TGUIIconListAbstract);
begin
  SetIconListForGUI(iconList);
end;

function SALHOTKEY(vk: Byte; mods: Word): DWORD;
begin
  Result:= (vk or (mods shl 8));
end;

end.

