unit spl_com;

interface

uses
  Windows,
  CommCtrl,
  winliblt;

const
  ICONOVERLAYINDEX_NOTUSED = 15;

  VALID_DATA_EXTENSION = $0001;
  VALID_DATA_DOSNAME = $0002;
  VALID_DATA_SIZE = $0004;
  VALID_DATA_TYPE = $0008;
  VALID_DATA_DATE = $0010;
  VALID_DATA_TIME = $0020;
  VALID_DATA_ATTRIBUTES = $0040;
  VALID_DATA_HIDDEN = $0080;
  VALID_DATA_ISLINK = $0100;
  VALID_DATA_ISOFFLINE = $0200;
  VALID_DATA_PL_SIZE = $0400;
  VALID_DATA_PL_DATE = $0800;
  VALID_DATA_PL_TIME = $1000;
  VALID_DATA_NONE = 0;
{$ifdef INSIDE_SALAMANDER}
  VALID_DATA_ALL = $FFFF;
{$endif}

  SALDIRFLAG_CASESENSITIVE = $0001;
  SALDIRFLAG_IGNOREDUPDIRS = $0002;
  VIEW_MODE_TREE = 1;
  VIEW_MODE_BRIEF = 2;
  VIEW_MODE_DETAILED = 3;
  VIEW_MODE_ICONS = 4;
  VIEW_MODE_THUMBNAILS = 5;
  VIEW_MODE_TILES = 6;

  TRANSFER_BUFFER_MAX = 1024;

  COLUMN_NAME_MAX = 30;
  COLUMN_DESCRIPTION_MAX = 100;

  COLUMN_ID_CUSTOM = 0;
  COLUMN_ID_NAME = 1;
  COLUMN_ID_EXTENSION = 2;
  COLUMN_ID_DOSNAME = 3;
  COLUMN_ID_SIZE = 4;
  COLUMN_ID_TYPE = 5;
  COLUMN_ID_DATE = 6;
  COLUMN_ID_TIME = 7;
  COLUMN_ID_ATTRIBUTES = 8;
  COLUMN_ID_DESCRIPTION = 9;

type
  TSalamanderRegistryAbstract = class
    function ClearKey(key: HKEY): BOOL; virtual; stdcall; abstract;
    function CreateKey(key: HKEY; name: PAnsiChar; var createdKey: HKEY): BOOL; virtual; stdcall; abstract;
    function OpenKey(key: HKEY; name: PAnsiChar; var openedKey: HKEY): BOOL; virtual; stdcall; abstract;
    procedure CloseKey(key: HKEY); virtual; stdcall; abstract;
    function DeleteKey(key: HKEY; name: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function GetValue(key: HKEY; name: PAnsiChar; _type: DWORD; buffer: Pointer; bufferSize: DWORD): BOOL; virtual; stdcall; abstract;
    function SetValue(key: HKEY; name: PAnsiChar; _type: DWORD; data: Pointer; dataSize: DWORD): BOOL; virtual; stdcall; abstract;
    function DeleteValue(key: HKEY; name: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function GetSize(key: HKEY; name: PAnsiChar; _type: DWORD; var bufferSize: DWORD): BOOL; virtual; stdcall; abstract;
  end;

  TQuadWord = Int64;
  PQuadWord = ^TQuadWord;

  TFileData = packed record
    Name: PAnsiChar;
    Ext: PAnsiChar;
    Size: TQuadWord;
    Attr: DWORD;
    LastWrite: TFileTime;
    DosName: PAnsiChar;
    PluginData: DWORD;
    NameLen: Cardinal;
    Hidden: Cardinal;
    IsLink: Cardinal;
    IsOffline: Cardinal;
    Association: Cardinal;
    Selected: Cardinal;
    Shared: Cardinal;
    Archive: Cardinal;
    SizeValid: Cardinal;
    Dirty: Cardinal;
    CutToClip: Cardinal;
    IconOverlayDone: Cardinal;
    IconOverlayIndex: Cardinal;
  end;
  PFileData = ^TFileData;

  TPluginDataInterfaceAbstract = class;

  TSalamanderDirectoryAbstract = class
    procedure Clear(pluginData: TPluginDataInterfaceAbstract); virtual; stdcall; abstract;
    procedure SetValidData(validData: DWORD); virtual; stdcall; abstract;
    procedure SetFlags(flags: DWORD); virtual; stdcall; abstract;
    function AddFile(path: PAnsiChar; var _file: TFileData; pluginData: TPluginDataInterfaceAbstract): BOOL; virtual; stdcall; abstract;
    function AddDir(path: PAnsiChar; var dir: TFileData; pluginData: TPluginDataInterfaceAbstract): BOOL; virtual; stdcall; abstract;
    function GetFilesCount: Integer; virtual; stdcall; abstract;
    function GetDirsCount: Integer; virtual; stdcall; abstract;
    function GetFile(index: Integer): PFileData; virtual; stdcall; abstract;
    function GetDir(index: Integer): PFileData; virtual; stdcall; abstract;
    function GetSalDir(index: Integer): TSalamanderDirectoryAbstract; virtual; stdcall; abstract;
    procedure SetApproximateCount(files: Integer; dirs: Integer); virtual; stdcall; abstract;
  end;

  TSalEnumSelectionFunc = function(enumFiles: Integer; isDir: PBOOL; size: PLargeInteger; fileData: PFileData; param: Pointer): PAnsiChar; stdcall;
  TSalEnumSelection2Func = function(parent: HWND; enumFiles: Integer; dosName: PAnsiChar; isDir: PBOOL; size: PLargeInteger; attr: PDWORD; lastWrite: PFileTime; param: Pointer; errorOccured: PBOOL): PAnsiChar; stdcall;

  TColumnGetTextProc = procedure; stdcall;

  TGetPluginIconIndexFunc = function: Integer; stdcall;

  TColumn = packed record
    Name: array[0..COLUMN_NAME_MAX] of AnsiChar;
    Description: array[0..COLUMN_DESCRIPTION_MAX] of AnsiChar;
    GetText: TColumnGetTextProc;
    CustomData: DWORD;
    SupportSorting: Cardinal;
    LeftAlignment: Cardinal;
    ID: Cardinal;
    Width: Cardinal;
    FixedWidth: Cardinal;
    MinWidth: Cardinal;
  end;
  PColumn = ^TColumn;

  TSalamanderViewAbstract = class
    function GetViewMode: DWORD; virtual; stdcall; abstract;
    procedure SetViewMode(viewMode: DWORD; validData: DWORD); virtual; stdcall; abstract;
    procedure GetTransferVariables(var transferFileData: PFileData; var transferIsDir: PInteger; var transferBuffer: PAnsiChar; var transferLen: PInteger; var transferRowData: PDWORD; var transferPluginDataIface: TPluginDataInterfaceAbstract; var transferActCustomData: PDWORD); virtual; stdcall; abstract;
    procedure SetPluginSimpleIconCallback(callback: TGetPluginIconIndexFunc); virtual; stdcall; abstract;
    function GetColumnsCount: Integer; virtual; stdcall; abstract;
    function GetColumn(index: Integer): PColumn; virtual; stdcall; abstract;
    function InsertColumn(index: Integer; column: PColumn): BOOL; virtual; stdcall; abstract;
    function InsertStandardColumn(index: Integer; id: DWORD): BOOL; virtual; stdcall; abstract;
    function SetColumnName(index: Integer; name: PAnsiChar; description: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function DeleteColumn(index: Integer): BOOL; virtual; stdcall; abstract;
  end;

  TPluginDataInterfaceAbstract = class
    function CallReleaseForFiles: BOOL; virtual; stdcall; abstract;
    function CallReleaseForDirs: BOOL; virtual; stdcall; abstract;
    procedure ReleasePluginData(var _file: TFileData; isDir: BOOL); virtual; stdcall; abstract;
    procedure GetFileDataForUpDir(archivePath: PAnsiChar; var upDir: TFileData); virtual; stdcall; abstract;
    function GetFileDataForNewDir(dirName: PAnsiChar; var dir: TFileData): BOOL; virtual; stdcall; abstract;
    function GetSimplePluginIcons(iconSize: Integer): HIMAGELIST; virtual; stdcall; abstract;
    function HasSimplePluginIcon(var _file: TFileData; isDir: BOOL): BOOL; virtual; stdcall; abstract;
    function GetPluginIcon(_file: PFileData; iconSize: Integer; var destroyIcon: BOOL): HICON; virtual; stdcall; abstract;
    function CompareFilesFromFS(file1: PFileData; file2: PFileData): Integer; virtual; stdcall; abstract;
    procedure SetupView(leftPanel: BOOL; view: TSalamanderViewAbstract; archivePath: PAnsiChar; upperDir: PFileData); virtual; stdcall; abstract;
    procedure ColumnFixedWidthShouldChange(leftPanel: BOOL; column: PColumn; newFixedWidth: Integer); virtual; stdcall; abstract;
    procedure ColumnWidthWasChanged(leftPanel: BOOL; column: PColumn; newWidth: Integer); virtual; stdcall; abstract;
    function GetInfoLineContent(panel: Integer; _file: PFileData; isDir: BOOL; selectedFiles: Integer; selectedDirs: Integer; displaySize: BOOL; var selectedSize: TQuadWord; buffer: PAnsiChar; hotTexts: PDWORD; var hotTextsCount: Integer): BOOL; virtual; stdcall; abstract;
    function CanBeCopiedToClipboard: BOOL; virtual; stdcall; abstract;
    function GetByteSize(_file: PFileData; isDir: BOOL; size: PLargeInteger): BOOL; virtual; stdcall; abstract;
    function GetLastWriteDate(_file: PFileData; isDir: BOOL; date: PSystemTime): BOOL; virtual; stdcall; abstract;
    function GetLastWriteTime(_file: PFileData; isDir: BOOL; time: PSystemTime): BOOL; virtual; stdcall; abstract;
  end;

  TSalamanderForOperationsAbstract = class
    procedure OpenProgressDialog(title: PAnsiChar; twoProgressBars: BOOL; parent: HWND; fileProgress: BOOL); virtual; stdcall; abstract;
    procedure ProgressDialogAddText(txt: PAnsiChar; delayedPaint: BOOL); virtual; stdcall; abstract;
    procedure ProgressSetTotalSize(var totalSize1: TQuadWord; var totalSize2: TQuadWord); virtual; stdcall; abstract;
    function ProgressSetSize(var size1: TQuadWord; var size2: TQuadWord; delayedPaint: BOOL): BOOL; virtual; stdcall; abstract;
    function ProgressAddSize(size: Integer; delayedPaint: BOOL): BOOL; virtual; stdcall; abstract;
    procedure ProgressEnableCancel(enable: BOOL); virtual; stdcall; abstract;
    function ProgressGetHWND: HWND; virtual; stdcall; abstract;
    procedure CloseProgressDialog; virtual; stdcall; abstract;
    function MoveFiles(source: PAnsiChar; target: PAnsiChar; remapNameFrom: PAnsiChar; remapNameTo: PAnsiChar): BOOL; virtual; stdcall; abstract;
  end;

(*
var
  SalamanderVersion: Integer;
*)

implementation

end.
