unit spl_crypt;

interface

uses
  Windows,
  winliblt;

const
  SAL_AES_MAX_KEY_LENGTH = 32;
  SAL_AES_MAX_PWD_LENGTH = 128;
  SAL_AES_MAX_SALT_LENGTH = 16;
  SAL_AES_PWD_VER_LENGTH = 2;

  SAL_AES_ERR_GOOD_RETURN = 0;
  SAL_AES_ERR_PASSWORD_TOO_LONG = -100;
  SAL_AES_ERR_BAD_MODE = -101;

  SAL_AES_BLOCK_SIZE = 16;
  SAL_AES_ENCR_CTX_SIZE = 264;
  SAL_AES_AUTH_CTX_SIZE = 160;

type
  TSalAES = packed record
   nonce: array[0..SAL_AES_BLOCK_SIZE - 1] of Byte;
   encr_bfr: array[0..SAL_AES_BLOCK_SIZE - 1] of Byte;
   encr_ctx: array[0..SAL_AES_ENCR_CTX_SIZE - 1] of Byte;
   auth_ctx: array[0..SAL_AES_AUTH_CTX_SIZE - 1] of Byte;
   encr_pos: DWORD;
   pwd_len: DWORD;
   mode: DWORD;
  end;
  PSalAES = ^TSalAES;

  TSalSHA1 = packed record
   state: array[0..4] of DWORD;
   count: array[0..1] of DWORD;
   buffer: array[0..63] of Byte;
  end;
  PSalSHA1 = ^TSalSHA1;

  TSalSHA1Digest = array[0..19] of Byte;

  TSalamanderCryptAbstract = class
    function AESInit(aes: PSalAES; mode: Integer; pwd: LPCSTR; pwd_len: size_t; salt: LPBYTE; pwd_ver: LPWORD): Integer; virtual; stdcall; abstract;
    procedure AESEncrypt(aes: PSalAES; data: Pointer; dataLen: size_t); virtual; stdcall; abstract;
    procedure AESDecrypt(aes: PSalAES; data: Pointer; dataLen: size_t); virtual; stdcall; abstract;
    procedure AESEnd(aes: PSalAES; mac: LPBYTE; pMacLen: LPDWORD); virtual; stdcall; abstract;
    procedure SHA1Init(sha1: TSalSHA1); virtual; stdcall; abstract;
    procedure SHA1Update(sha1: TSalSHA1; data: LPBYTE; dataLen: size_t); virtual; stdcall; abstract;
    procedure SHA1Final(sha1: TSalSHA1; digest: TSalSHA1Digest); virtual; stdcall; abstract;
  end;

implementation

end.
