unit spl_fs;

interface

uses
  Windows,
  winliblt,
  spl_com;

const
  pitSimple = 0;
  pitFromRegistry = 1;
  pitFromPlugin = 2;

  FSE_CLOSEORDETACHCANCELED = 0;
  FSE_OPENED = 1;
  FSE_DETACHED = 2;
  FSE_ATTACHED = 3;
  FSE_ACTIVATEREFRESH = 4;
  FSE_TIMER = 5;
  FSE_PATHCHANGED = 6;

  FSTRYCLOSE_CHANGEPATH = 1;
  FSTRYCLOSE_UNLOADCLOSEFS = 2;
  FSTRYCLOSE_CHANGEPATHFAILURE = 3;
  FSTRYCLOSE_ATTACHFAILURE = 4;
  FSTRYCLOSE_UNLOADCLOSEDETACHEDFS = 5;
  FSTRYCLOSE_PLUGINCLOSEDETACHEDFS = 6;
  FSTRYCLOSE_ERRORCLOSEDETACHEDFS = 7;

  FS_SERVICE_COPYFROMFS = $00000001;
  FS_SERVICE_MOVEFROMFS = $00000002;
  FS_SERVICE_COPYFROMDISKTOFS = $00000004;
  FS_SERVICE_MOVEFROMDISKTOFS = $00000008;
  FS_SERVICE_DELETE = $00000010;
  FS_SERVICE_QUICKRENAME = $00000020;
  FS_SERVICE_VIEWFILE = $00000040;
  FS_SERVICE_EDITFILE = $00000080;
  FS_SERVICE_EDITNEWFILE = $00000100;
  FS_SERVICE_CHANGEATTRS = $00000200;
  FS_SERVICE_CREATEDIR = $00000400;
  FS_SERVICE_SHOWINFO = $00000800;
  FS_SERVICE_SHOWPROPERTIES = $00001000;
  FS_SERVICE_CALCULATEOCCUPIEDSPACE = $00002000;
  FS_SERVICE_COMMANDLINE = $00008000;
  FS_SERVICE_GETFREESPACE = $00010000;
  FS_SERVICE_GETFSICON = $00020000;
  FS_SERVICE_GETNEXTDIRLINEHOTPATH = $00040000;
  FS_SERVICE_CONTEXTMENU = $00080000;
  FS_SERVICE_GETCHANGEDRIVEORDISCONNECTITEM = $00100000;
  FS_SERVICE_ACCEPTSCHANGENOTIF = $00200000;
  FS_SERVICE_GETPATHFORMAINWNDTITLE = $00400000;
  FS_SERVICE_OPENFINDDLG = $00800000;
  FS_SERVICE_OPENACTIVEFOLDER = $01000000;
  FS_SERVICE_SHOWSECURITYINFO = $02000000;

  fscmItemsInPanel = 0;
  fscmPathInPanel = 1;
  fscmPanel = 2;
  
  SALCMDLINE_MAXLEN = 8192;

type
  TSalamanderForViewFileOnFSAbstract = class
    function AllocFileNameInCache(parent: HWND; uniqueFileName: PAnsiChar; nameInCache: PAnsiChar; rootTmpPath: PAnsiChar; var fileExists: BOOL): PAnsiChar; virtual; stdcall; abstract;
    function OpenViewer(parent: HWND; fileName: PAnsiChar; fileLock: PHandle; fileLockOwner: PBOOL): BOOL; virtual; stdcall; abstract;
    procedure FreeFileNameInCache(uniqueFileName: PAnsiChar; fileExists: BOOL; newFileOK: BOOL; var newFileSize: TQuadWord; fileLock: THandle; fileLockOwner: BOOL; removeAsSoonAsPossible: BOOL); virtual; stdcall; abstract;
  end;

  TPluginFSInterfaceAbstract = class
    function GetCurrentPath(userPart: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function GetFullName(var _file: TFileData; isDir: Integer; buf: PAnsiChar; bufSize: Integer): BOOL; virtual; stdcall; abstract;
    function GetFullFSPath(parent: HWND; fsName: PAnsiChar; path: PAnsiChar; pathSize: Integer; var success: BOOL): BOOL; virtual; stdcall; abstract;
    function GetRootPath(userPart: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function IsCurrentPath(currentFSNameIndex: Integer; fsNameIndex: Integer; userPart: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function IsOurPath(currentFSNameIndex: Integer; fsNameIndex: Integer; userPart: PAnsiChar): BOOL; virtual; stdcall; abstract;
    function ChangePath(currentFSNameIndex: Integer; fsName: PAnsiChar; fsNameIndex: Integer;userPart: PAnsiChar; cutFileName: PAnsiChar; pathWasCut: PBOOL;forceRefresh: BOOL; mode: Integer): BOOL; virtual; stdcall; abstract;
    function ListCurrentPath(dir: TSalamanderDirectoryAbstract; var pluginData: TPluginDataInterfaceAbstract; var iconsType: Integer; forceRefresh: BOOL): BOOL; virtual; stdcall; abstract;
    function TryCloseOrDetach(forceClose: BOOL; canDetach: BOOL; var detach: BOOL; reason: Integer): BOOL; virtual; stdcall; abstract;
    procedure Event(event: Integer; param: DWORD); virtual; stdcall; abstract;
    procedure ReleaseObject(parent: HWND); virtual; stdcall; abstract;
    function GetSupportedServices: DWORD; virtual; stdcall; abstract;
    function GetChangeDriveOrDisconnectItem(fsName: PAnsiChar; var title: PAnsiChar; var icon: HICON; var destroyIcon: BOOL): BOOL; virtual; stdcall; abstract;
    function GetFSIcon(var destroyIcon: BOOL): HICON; virtual; stdcall; abstract;
    procedure GetDropEffect(srcFSPath: PAnsiChar; tgtFSPath: PAnsiChar; allowedEffects: DWORD; keyState: DWORD; dropEffect: PDWORD); virtual; stdcall; abstract;
    procedure GetFSFreeSpace(retValue: PLargeInteger); virtual; stdcall; abstract;
    function GetNextDirectoryLineHotPath(text: PAnsiChar; pathLen: Integer; var offset: Integer): BOOL; virtual; stdcall; abstract;
    procedure CompleteDirectoryLineHotPath(path: PAnsiChar; pathBufSize: Integer); virtual; stdcall; abstract;
    function GetPathForMainWindowTitle(fsName: PAnsiChar; mode: Integer; buf: PAnsiChar; bufSize: Integer): BOOL; virtual; stdcall; abstract;
    procedure ShowInfoDialog(fsName: PAnsiChar; parent: HWND); virtual; stdcall; abstract;
    function ExecuteCommandLine(parent: HWND; command: PAnsiChar; var selFrom: Integer; var selTo: Integer): BOOL; virtual; stdcall; abstract;
    function QuickRename(fsName: PAnsiChar; mode: Integer; parent: HWND; var _file: TFileData; isDir: BOOL; newName: PAnsiChar; var cancel: BOOL): BOOL; virtual; stdcall; abstract;
    procedure AcceptChangeOnPathNotification(fsName: PAnsiChar; path: PAnsiChar; includingSubdirs: BOOL); virtual; stdcall; abstract;
    function CreateDir(fsName: PAnsiChar; mode: Integer; parent: HWND; newName: PAnsiChar; var cancel: BOOL): BOOL; virtual; stdcall; abstract;
    procedure ViewFile(fsName: PAnsiChar; parent: HWND; salamander: TSalamanderForViewFileOnFSAbstract; var _file: TFileData); virtual; stdcall; abstract;
    function Delete(fsName: PAnsiChar; mode: Integer; parent: HWND; panel: Integer; selectedFiles: Integer; selectedDirs: Integer; var cancelOrError: BOOL): BOOL; virtual; stdcall; abstract;
    function CopyOrMoveFromFS(copy: BOOL; mode: Integer; fsName: PAnsiChar; parent: HWND; panel: Integer; selectedFiles: Integer; selectedDirs: Integer; targetPath: PAnsiChar; var operationMask: BOOL; var cancelOrHandlePath: BOOL; dropTarget: HWND): BOOL; virtual; stdcall; abstract;
    function CopyOrMoveFromDiskToFS(copy: BOOL; mode: Integer; fsName: PAnsiChar; parent: HWND; sourcePath: PAnsiChar; next: TSalEnumSelection2Func; nextParam: Pointer; sourceFiles: Integer; sourceDirs: Integer; targetPath: PAnsiChar; invalidPathOrCancel: PBOOL): BOOL; virtual; stdcall; abstract;
    function ChangeAttributes(fsName: PAnsiChar; parent: HWND; panel: Integer; selectedFiles: Integer; selectedDirs: Integer): BOOL; virtual; stdcall; abstract;
    procedure ShowProperties(fsName: PAnsiChar; parent: HWND; panel: Integer; selectedFiles: Integer; selectedDirs: Integer); virtual; stdcall; abstract;
    procedure ContextMenu(fsName: PAnsiChar; parent: HWND; menuX: Integer; menuY: Integer; _type: Integer; panel: Integer; selectedFiles: Integer; selectedDirs: Integer); virtual; stdcall; abstract;
    function HandleMenuMsg(uMsg: UINT; wParam: WPARAM; lParam: LPARAM): BOOL; virtual; stdcall;
    function HandleMenuMsg2(uMsg: UINT; wParam: WPARAM; lParam: LPARAM; plResult: PLRESULT): BOOL; virtual; stdcall; abstract;
    function OpenFindDialog(fsName: PAnsiChar; panel: Integer): BOOL; virtual; stdcall; abstract;
    procedure OpenActiveFolder(fsName: PAnsiChar; parent: HWND; useOpenNotExplore: BOOL); virtual; stdcall; abstract;
    procedure GetAllowedDropEffects(mode: Integer; tgtFSPath: PAnsiChar; allowedEffects: PDWORD); virtual; stdcall; abstract;
    function GetNoItemsInPanelText(textBuf: PAnsiChar; textBufSize: Integer): BOOL; virtual; stdcall; abstract;
    function ShowSecurityInfo(parent: HWND): BOOL; virtual; stdcall; abstract;
  end;

  TPluginInterfaceForFSAbstract = class
    function OpenFS(fsName: PAnsiChar; fsNameIndex: Integer): TPluginFSInterfaceAbstract; virtual; stdcall; abstract;
    procedure CloseFS(fs: TPluginFSInterfaceAbstract); virtual; stdcall; abstract;
    procedure ExecuteChangeDriveMenuItem; virtual; stdcall;
    procedure ExecuteChangeDriveMenuItem2(Panel: Integer); virtual; stdcall; abstract;
    function ChangeDriveMenuItemContextMenu(parent: HWND; x: Integer; y: Integer; pluginFS: TPluginFSInterfaceAbstract; pluginFSName: PAnsiChar; pluginFSNameIndex: Integer; isDetachedFS: BOOL; var refreshMenu: BOOL; var closeMenu: BOOL; var postCmd: Integer; var postCmdParam: Pointer): BOOL; virtual; stdcall;
    function ChangeDriveMenuItemContextMenu2(parent: HWND; panel: Integer; x: Integer; y: Integer; pluginFS: TPluginFSInterfaceAbstract; pluginFSName: PAnsiChar; pluginFSNameIndex: Integer; isDetachedFS: BOOL; var refreshMenu: BOOL; var closeMenu: BOOL; var postCmd: Integer; var postCmdParam: Pointer): BOOL; virtual; stdcall; abstract;
    procedure ExecuteChangeDrivePostCommand(postCmd: Integer; postCmdParam: Pointer); virtual; stdcall;
    procedure ExecuteChangeDrivePostCommand2(panel: Integer; postCmd: Integer; postCmdParam: Pointer); virtual; stdcall; abstract;
    procedure ExecuteOnFS(panel: Integer; pluginFS: TPluginFSInterfaceAbstract; pluginFSName: PAnsiChar; pluginFSNameIndex: Integer; var _file: TFileData; isDir: Integer); virtual; stdcall; abstract;
    function DisconnectFS(parent: HWND; isInPanel: BOOL; panel: Integer; pluginFS: TPluginFSInterfaceAbstract; pluginFSName: PAnsiChar; pluginFSNameIndex: Integer): BOOL; virtual; stdcall; abstract;
    procedure ConvertPathToInternal(fsName: PAnsiChar; fsNameIndex: Integer; fsUserPart: PAnsiChar); virtual; stdcall; abstract;
    procedure ConvertPathToExternal(fsName: PAnsiChar; fsNameIndex: Integer; fsUserPart: PAnsiChar); virtual; stdcall; abstract;
    procedure EnsureShareExistsOnServer(panel: Integer; server: PAnsiChar; share: PAnsiChar); virtual; stdcall; abstract;
  end;

implementation

uses
  spl_gen;

function TPluginFSInterfaceAbstract.HandleMenuMsg(uMsg: UINT; wParam: WPARAM; lParam: LPARAM): BOOL;
var
  _lResult: LRESULT;
begin
  Result := HandleMenuMsg2(uMsg, wParam, lParam, @_lResult);
end;

procedure TPluginInterfaceForFSAbstract.ExecuteChangeDriveMenuItem;
begin
  ExecuteChangeDriveMenuItem2(PANEL_SOURCE);
end;

function TPluginInterfaceForFSAbstract.ChangeDriveMenuItemContextMenu(parent: HWND; x: Integer; y: Integer; pluginFS: TPluginFSInterfaceAbstract; pluginFSName: PAnsiChar; pluginFSNameIndex: Integer; isDetachedFS: BOOL; var refreshMenu: BOOL; var closeMenu: BOOL; var postCmd: Integer; var postCmdParam: Pointer): BOOL;
begin
  Result := ChangeDriveMenuItemContextMenu2(parent, PANEL_SOURCE, x, y, pluginFS, pluginFSName, pluginFSNameIndex, isDetachedFS, refreshMenu, closeMenu, postCmd, postCmdParam);
end;

procedure TPluginInterfaceForFSAbstract.ExecuteChangeDrivePostCommand(postCmd: Integer; postCmdParam: Pointer);
begin
  ExecuteChangeDrivePostCommand2(PANEL_SOURCE, postCmd, postCmdParam);
end;

end.
